/*-
 * Copyright (c) 2008 Andrey V. Surkov <nsand@sura.ru>
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 *	 $Id: my_types.h,v 0.52 2008/05/23 07:54:36 nsand Exp $
 */


#include <curses.h>
#include <panel.h>
//#include <net/ethernet.h>
#include <signal.h> 
#include <pthread.h> 
#include <netinet/in.h>
#include <netinet/in_systm.h>

#ifndef _SKYFISH_TYPE_
#define	_SKYFISH_TYPE_

#if !defined(NCURSES_VERSION)
#error "Not found library NCURSES please install it"
#endif
#define KOI8R	0
#define CP866	1
#define CP1251	2


static char VERSION[] = "0.90";

#define	LENTMP 320
#define	LENPATH 300
#define TIME_CHECK 1
#define SRCDSTSIZE 300		/*for src dst path name*/
#define TYPE_HTTP 0x50545448
#define READBUFSIZE 0xffff + ETHSIZE
#define MAXIFACE 15
#define DBTSIZE 100		/* buffer size filterdbt */
#define ETHSIZE sizeof(struct ether_header)
#define IP_H  sizeof(struct ip)
#define TCP_H   sizeof(struct tcphdr)
#define ClearTMP() { memset(tmp,0,LENTMP); }
#define FILTERBUFFER 	100
#define SERVER 1
#define CLIENT 2
#define ALONE  3
struct filter *filterdb;
/* storage all known type and size for filtering */
struct filter {
    u_int64_t min;
    u_int64_t max;
    char type[32];
};

/* for filtering text in HTTP header */
struct filterdbt {
    char search [100];
    char type[32];
};

/* for filtering in file (binary mode) */
struct filterdbb {
    u_char search [50];
    char type[32];
    u_long len;
};

struct share_value {
//	pthread_rwlock_t *rwlock;
	u_int64_t alltraf; 
	u_int64_t writraf; 
	u_int64_t badtraf; 
	u_long countpks;
	u_long discount;
	u_long over;
	u_long openfiles;
	u_long realspeed;
	u_long speed;
	int codepage[2];
};

/* stream structure */
struct pks {
    u_int64_t X;				// 64 bit length file ii Content-Range
    u_int64_t len;			// 64 bit length file
    u_int64_t lencur;			// 64 bit summ of bytes    
    u_long dst;				// 32 bit IP
    u_long src;				// 32 bit IP
    u_short port_d;			// 16 bit TCP
    u_short port_s;			// 16 bit TCP
    u_long ack;				// 32 bit TCP
    u_long http_offset;			// offset for begin file
    u_long start_count;			/* 32 bit TCP = ack 8 */
    u_long refer;			/* refer to main stream if present */
    u_long numfx;			/* count [1-3][5-7] */
    time_t atime;			/* last acces time to file */
    time_t ctime;			/* time to create file */
    time_t deltatime;			/* last acces in file in seconds */
    u_char begin;			/* key is 0- not begin or 200 - begin file */
    char http[3001];			/* fo storage header HTTP */
    u_long  httpf;			/* key for all data in http */
    char name[250];
    char findname[250];			/* if name found in HTTP */
    char type[32];			/* type file */
#define MASSIZE 250
    u_int64_t mas[MASSIZE];  		/* buffer for define fullness file size*/
    char etag[64];			/* Etag in HTTP protocol */
    int fd;
    int fordel;				/* mark for delete */
	u_long rspeed;
	u_long countspeed;
};

struct crossxy {
	u_int64_t x;
	u_int64_t y;
	int data;
};


#define SIZEOF64 sizeof(u_int64_t)
#define PKS sizeof(struct pks)

#define	FILE_MODE	(S_IRUSR | S_IWUSR | S_IRGRP | S_IROTH)
					/* default permissions for new files */
#include	<sys/types.h>	/* basic system data types */
#include	<sys/time.h>	/* timeval{} for select() */
#include	<time.h>		/* timespec{} for pselect() */
#include	<errno.h>
#include	<fcntl.h>		/* for nonblocking */
#include	<limits.h>		/* PIPE_BUF */
#include	<signal.h>
#include	<stdio.h>
#include	<stdlib.h>
#include	<string.h>
#include	<sys/stat.h>	/* for S_xxx file mode constants */
#include	<unistd.h>
#include	<sys/wait.h>

#ifdef	HAVE_MQUEUE_H
# include	<mqueue.h>		/* Posix message queues */
#endif

#ifdef	HAVE_SEMAPHORE_H
# include	<semaphore.h>	/* Posix semaphores */
#endif


#ifdef	HAVE_SYS_MMAN_H
# include	<sys/mman.h>	/* Posix shared memory */
#endif

#ifndef	MAP_FAILED
#define	MAP_FAILED	((void *)(-1))
#endif

#ifdef	HAVE_SYS_IPC_H
# include	<sys/ipc.h>		/* System V IPC */
#endif

#ifdef	HAVE_SYS_MSG_H
# include	<sys/msg.h>		/* System V message queues */
#endif

//#ifdef	HAVE_SYS_SEM_H
//#ifdef	__bsdi__
//#undef	HAVE_SYS_SEM_H		/* hack: BSDI's semctl() prototype is wrong */
//#else
//# include	<sys/sem.h>		/* System V semaphores */
//#endif

//#ifndef	HAVE_SEMUN_UNION
/* $$.It semun$$ */
//union semun {				/* define union for semctl() */
//  int              val;
//  struct semid_ds *buf;
//  unsigned short  *array;
//};
//#endif
//#endif	/* HAVE_SYS_SEM_H */

//#ifdef	HAVE_SYS_SHM_H
//# include	<sys/shm.h>		/* System V shared memory */
//#endif

//#ifdef	HAVE_SYS_SELECT_H
//# include	<sys/select.h>	/* for convenience */
//#endif

//#ifdef	HAVE_STROPTS_H
//# include	<stropts.h>		/* for convenience */
//#endif

//#ifdef	HAVE_SYS_MMAN_H
#include	<sys/mman.h>	/* Posix shared memory */
//#endif

#endif 
