/*-
 * Copyright (c) 2008 Andrey V. Surkov <nsand@sura.ru>
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 *	 $Id: skyfish.c,v 0.52 2008/05/23 07:54:36 nsand Exp $
 */
 
#include <string.h>
#include <errno.h>
#include <signal.h>
#include <stdlib.h>
#include <stdio.h>
#include <unistd.h>
#include <fcntl.h>
#include <time.h>

#include <sys/ioctl.h>
#include <sys/time.h>
#include <sys/stat.h>
#include <sys/event.h>


#include <netinet/in.h>
#include <netinet/in_systm.h>
#include <netinet/ip.h>
#include <netinet/ip_var.h>
#include <netinet/tcp.h>
#include <netinet/tcp_var.h>
#include <sys/socket.h> //socket()   

#include <ncurses.h> 

#include "skyfish.h"
#include "sky_shm.h"

#ifndef NTOHS
#define NTOHS(a) (a) = ntohs((a))
#endif
u_long MAXTHREADS;
char 	tmp[LENTMP];
//u_char	FAST_SCREEN;			/* fast refresh screen */

int done;
char ifaces[MAXIFACE][100];
char *readdev;
struct sigaction action;
struct sigaction chwindow;
struct sigaction daemonexit;
struct sigaction daemoninfo;
int		runmode;
u_long frpks;
struct pks *db;
struct share_value *sharedb;
struct filterdbt *dbt;
struct filterdbb *dbb;
uint32_t blockdb[100];  // for testing
uint32_t countblock;
int filtercount;

int
main(int argc, char **argv)
{
	FILE *fd;
	pid_t parpid;
	pid_t rpid;
	int ch;

	runmode= ALONE;
	if ((fd = fopen("/var/run/skyfish.pid","r")) != NULL){
		fscanf(fd,"%d",&rpid);
		fclose(fd);
		}

	while ((ch = getopt(argc, argv, "rdsi")) != -1)
		switch (ch) {
		case 'd': /* daemon mode */
			if (rpid > 0) {printf("Daemon already running\n");return 1;}
			runmode=SERVER;
			if((parpid=fork())<0) {	                  
					printf("\ncan't fork");
					exit(1);               
			}else if (parpid!=0) exit(0);
			setsid(); 
			chdir("/");
			close(0);
			close(1);
			close(2);
			break;
		case 's':
			if (rpid == 0) {printf("No running daemon\n");return 1;}
			kill(rpid,SIGUSR1);printf("Send signal to stop!\n"); return 0;
			break;
		case 'i':
			if (rpid == 0) {printf("No running daemon\n");return 1;}
			kill(rpid,SIGUSR2);printf("Send signal to save in log!\n"); return 0;
			break;
		case 'r':
			runmode=CLIENT;
			if (rpid == 0) {printf("No running daemon\n");return 1;}
			break;
		default:
			printf("read man skyfish\n");
			break;
		}
	if (argc < 2 && rpid > 0) {printf("Daemon already running\n");return 1;} 
	if (runprog() == 1) printf("error running programm!\n");
	return 0;
}


int
runprog(void)
{
   FILE *fdp;
   pid_t parpid=getpid();
   char pids[10];
		sprintf(pids,"%d",parpid);	
	if (runmode != CLIENT) {
		fdp = fopen("/var/run/skyfish.pid","w");
		fwrite(pids,strlen(pids),1,fdp);
		fclose(fdp);
	}

    if (runmode != SERVER) if (init_scr()) 	return 1;
    if (initialize()) 	{ remove_pid();return 1;}
	if (runmode != CLIENT) {
    if (initnode())  	{ remove_pid();return 1;}
	ShutNodes();
    if (AddNodes()) 	{ remove_pid();return 1;}
    CloseSocket();
    if (CheckPaths()) 	{ remove_pid();return 1;}
}
    int fd, kq, n=0;
    struct timespec ts = {1, 0};    /*  0      */
    struct timespec twait = {0, 5000};    /*        */

    u_long i;
	if (runmode != CLIENT)
    for (i = 0;i < MAXTHREADS; i++) {
	ClearPKS(i);
    }

if (runmode != CLIENT )
{
    fd = open("/dev/skyfish", O_RDONLY, 0);
    if (fd == -1) {
	printf("Can't open /dev/skyfish \n");
	return 1;
    }
}
    kq = kqueue();
    if (kq == -1)
	printf("error get kqueue\n");
	      
    struct kevent change;
    struct kevent event;    

    EV_SET(&change, fd, EVFILT_READ, EV_ADD | EV_ENABLE, 0, 0, NULL);//isok());

    time_t curt = 0, oldt=0;

    int ch;    
	int rd;
    int isevent = 0;
	uint32_t *over;
    over=0;
        if (RegisterTimer() != 0) 	return 1;
    done = 1;	
while(done)
{
	if (runmode != CLIENT) isevent = kevent(kq, &change, 1, &event, 1, &ts);
	if (runmode != SERVER) {
		ch=getch();
		if (ch !=  ERR) CheckKey(ch);
		else nanosleep(&twait,NULL);
		show_table();
		if (runmode == CLIENT) continue;
	}
		if (isevent > 0)// {
	    	for (i = 0; i < event.data;i++)  {
				bzero(readdev,READBUFSIZE);
				rd = read(fd, readdev,READBUFSIZE);
				if (rd > 4) {
					handler(readdev);
					}  else if (rd = 4){ over = (uint32_t*)(readdev);
							sharedb->over += *over;
							*over = 0;
							i--;
						}
			}	
		time(&curt);
		if ((curt - oldt) >= TIME_CHECK) { 
			oldt = curt; 
			CheckTime(); 	//move to timer clock
		}		
}
    close(kq);	
    close(fd);	
    CloseProg();
    return 0;	
}


/*
*   main handler received packet
*/
int 
handler(const char* packet)
{
    struct ip *ip  = (struct ip *) (packet + ETHSIZE);
    int l = ip->ip_hl<<2;
    u_long sum =(ip->ip_sum);
    ip->ip_sum=0;
    u_short chksum = checksum((const u_char*)(packet + ETHSIZE), l, sum);
    ip->ip_sum=sum;

    u_short chk_sumip = in_cksumip((u_short *)(packet + ETHSIZE),  ip->ip_hl<<2, ip->ip_sum);
    if (chksum != 0)  { 
		sharedb->discount ++;
		return 1;
    }

    u_long lenip = ntohs(ip->ip_len);
    u_long len_tcp = lenip - (ip->ip_hl<<2);
    sharedb->speed   += lenip;
    AddAlltraf(lenip);
    u_short ipoff = ntohs(ip->ip_off);
    
    if ((ipoff & IP_OFFMASK) || (ipoff & IP_MF))
    {
     return 1;
    }

    if (ip->ip_p != IPPROTO_TCP) return 0;	

    struct tcphdr *tcp	= (struct tcphdr *)(packet + (ip->ip_hl<<2) + ETHSIZE);	

    u_short chk_sum  = tcp_cksum(ip->ip_src, ip->ip_dst, ip->ip_p,len_tcp,(const char *)tcp, tcp->th_sum);
    if (chk_sum != tcp->th_sum && chk_sum != 0 && tcp->th_sum != 0) {
	sharedb->discount++;
	return 1;
    }

    if ((len_tcp - (tcp->th_off<<2)) == 0) return 1;
    NTOHS(ip->ip_len);

	uint32_t i;
	for (i=0; i<countblock; i++)
		if (ip->ip_src.s_addr == blockdb[i]) return 1;

    if (ip->ip_p == IPPROTO_TCP) parse_TCP(packet);
return 0;
} 


/*
*  initialise vars from config file
*/
int 
initialize(void)
{
	int flags = O_RDWR | O_CREAT;
	u_long length = sizeof(struct share_value);
	int fdf = Shm_open("/tmp/sharedb.shm", flags, FILE_MODE);
	Ftruncate(fdf, length);
	sharedb = Mmap(NULL, length, PROT_READ | PROT_WRITE, MAP_SHARED, fdf, 0);
	Close(fdf);
	
//	if (pthread_rwlock_init(sharedb->rwlock,NULL)!=0) return 1;//PTHREAD_PROCESS_SHARED);

    if (runmode != CLIENT ) { 
		sharedb->countpks 	= 0;
		sharedb->badtraf = 0;		
		sharedb->writraf = 0;		
		sharedb->discount 	= 0;
		sharedb->over 	= 0;
		sharedb->speed 	= 0;
	}
    SetRealspeed(0);
//    FAST_SCREEN = 1;
    MAXTHREADS 	= 100;
    SetTLB(10);
SetPathLog("./log.txt");
SetPathOk("./ok");
SetPathTmp("./temp");    
SetPathInc("./incomplete");
char codepagefrom[10];
char codepageto[10];
	
	flags = O_RDWR | O_CREAT;
	length = FILTERBUFFER * sizeof(struct filter);
	fdf = Shm_open("/tmp/filterdb.shm", flags, FILE_MODE);
	Ftruncate(fdf, length);
	filterdb = Mmap(NULL, length, PROT_READ | PROT_WRITE, MAP_SHARED, fdf, 0);
	Close(fdf);
	bzero(&filterdb[0],length);

	sharedb->codepage[0]=255; // default use locale codepage
    dbt = (struct filterdbt*)malloc(DBTSIZE * sizeof(struct filterdbt)); // 
	if (dbt == NULL) 	{ printf("calloc dbt error\n"); 	return 1;}

    dbb = (struct filterdbb*)malloc(DBTSIZE * sizeof(struct filterdbb)); // 
	if (dbb == NULL) 	{ printf("calloc dbb error\n"); 	return 1;}

    bzero(dbb,DBTSIZE * sizeof(struct filterdbb));
    int i=0;
    char var[100];
    u_int64_t value;
    FILE *fd;
        if ((fd = fopen("/usr/local/etc/skyfish.conf","r")) == NULL){
	    printf("Not found file /usr/local/etc/skyfish.conf\n");
	    return 1;
    }
    int rd = 0;
    int done = 0;
    u_long n;
	u_long a,b,c,d,block;
	char tm;
    bzero(var,100);
    while (done == 0) {
	if (strlen(var) == 0) rd = fscanf(fd,"%s",var);
	    else {printf("error format file skyfish.conf var=%s\n",var); done=1;return 1;}
	if (!strcmp(var,"end")) done = 1;
	if (rd = 0) done = 1;
	if (!strcmp(var,"TIME_LIVE_THREAD")) 	{fscanf(fd,"%lu",&n); SetTLT(n); memset(var,0,100);}
	if (!strcmp(var,"TIME_END_FILE")) 	{fscanf(fd,"%lu",&n); SetTEF(n); memset(var,0,100);}
	if (!strcmp(var,"NOCOMPLETE")) 		{fscanf(fd,"%lu",&n);	SetNocomplete(n);	memset(var,0,100);}
	if (!strcmp(var,"MAXTHREADS")) 		{fscanf(fd,"%lu",&MAXTHREADS);		memset(var,0,100);}
	if (!strcmp(var,"OK")) 			{ClearTMP();fscanf(fd,"%s",tmp); SetPathOk(tmp);memset(var,0,100);}
	if (!strcmp(var,"#")) 			{ClearTMP();
		while (tm != '\n') 	fscanf(fd,"%c",&tm); 
		tm=0;
		memset(var,0,100);
		}
	if (!strcmp(var,"TEMP")) 		{ClearTMP();fscanf(fd,"%s",tmp); SetPathTmp(tmp);memset(var,0,100);}
	if (!strcmp(var,"INCOMPLETE")) 		{ClearTMP();fscanf(fd,"%s",tmp); SetPathInc(tmp);memset(var,0,100);}
	if (!strcmp(var,"block") && countblock<100) 		{ClearTMP();fscanf(fd,"%d.%d.%d.%d",&a,&b,&c,&d); blockdb[countblock] = htonl((a<<24) + (b<<16) + (c<<8) + d);countblock++; memset(var,0,100);}
	if (!strcmp(var,"version")) 			{memset(var,0,100);   fscanf(fd,"%s",var);	
	if (strcmp(var,VERSION)) {printf("Error skyfish.conf is other version!\n"); return 1;}memset(var,0,100);}
	if (!strcmp(var,"LOG")) 		{ClearTMP();fscanf(fd,"%s",tmp); SetPathLog(tmp);memset(var,0,100);}
	if (!strcmp(var,"iface")) 		{if (Getnumface() < MAXIFACE) fscanf(fd,"%s",ifaces[Getnumface()]);Addnumface();	memset(var,0,100);}	

	if (!strcmp(var,"CODEPAGEFILE")) 		{fscanf(fd,"%s",codepagefrom);fscanf(fd,"%s",codepageto);	memset(var,0,100);}	
	if (!strcmp(var,"^t")) {
	    for (i=0; i < DBTSIZE; i++)
		if (strlen(dbt[i].type) == 0) {
		    fscanf(fd,"%s",dbt[i].search); 
		    fscanf(fd,"%s",dbt[i].type);
		    i = DBTSIZE;
		}
	memset(var,0,100);		
	}
	if (!strcmp(var,"^b")) {
	    for (i=0; i < DBTSIZE; i++)
		if (strlen(dbb[i].type) == 0) {
		    fscanf(fd,"%s",dbb[i].search); 
		    fscanf(fd,"%s",dbb[i].type);
		    i = DBTSIZE;
		}
	memset(var,0,100);		
	}

	if (!strcmp(var,"type")) {
	    for (i=0; i < FILTERBUFFER; i++)
		if (strlen(filterdb[i].type) == 0) {
			filtercount++;
		    fscanf(fd,"%s",filterdb[i].type); //printf("type= %s \t",filetdb[i].type);
		    memset(var,0,100);
		    fscanf(fd,"%s",var);
			if (!strcmp(var,"-")) { 
				filterdb[i].min = 0; 
				filterdb[i].max = 0; 
				i = FILTERBUFFER; 
				memset(var,0,100);
				continue;
			} 

		    sscanf(var,"%llu",&filterdb[i].min);

			if (strcasestr(var,"kb")  != NULL) {filterdb[i].min *= KBYTEL;}
			else if (strcasestr(var,"mb")  != NULL) {filterdb[i].min *= MBYTEL;}
			else if (strcasestr(var,"gb")  != NULL) {filterdb[i].min *= GBYTEL;}			
			else if (strcasestr(var,"tb")  != NULL) {filterdb[i].min *= TBYTEL;}
			else { memset(var,0,100);}
		    if (filterdb[i].max == 0) fscanf(fd,"%s",var); 
		    sscanf(var,"%llu",&filterdb[i].max);
			if (strcasestr(var,"kb")  != NULL) {filterdb[i].max *= KBYTEL;memset(var,0,100);}
			else if (strcasestr(var,"mb")  != NULL) {filterdb[i].max *= MBYTEL;bzero(var,100);}
			else if (strcasestr(var,"gb")  != NULL) {filterdb[i].max *= GBYTEL;bzero(var,100);}			
			else if (strcasestr(var,"tb")  != NULL) {filterdb[i].max *= TBYTEL;bzero(var,100);}
		    if (rd = 0) done = 1;			
		    i = FILTERBUFFER;
		    }
	    	
	}
    }
    
    fclose(fd);
	if (!strcmp(codepagefrom,"KOI8R"))  sharedb->codepage[0] = 0;
	if (!strcmp(codepagefrom,"CP866"))  sharedb->codepage[0] = 1;
	if (!strcmp(codepagefrom,"CP1251")) sharedb->codepage[0] = 2;
   	if (!strcmp(codepageto,"KOI8R"))  sharedb->codepage[1] = 0;
	if (!strcmp(codepageto,"CP866"))  sharedb->codepage[1] = 1;
	if (!strcmp(codepageto,"CP1251")) sharedb->codepage[1] = 2;
 
    if (OpenLog() != 0) return 1;

    readdev = (char*) malloc(READBUFSIZE);
    if (readdev == NULL) 	{ printf("malloc readdev error\n"); 	return 1;}

	flags = O_RDWR | O_CREAT;
	length = MAXTHREADS*PKS;
	fdf = Shm_open("/tmp/db.shm", flags, FILE_MODE);
	Ftruncate(fdf, length);
	db = Mmap(NULL, length, PROT_READ | PROT_WRITE, MAP_SHARED, fdf, 0);
	Close(fdf);

    TransformDBB();
return 0;
}


u_short
in_cksumip(const u_short *addr, register u_int len, int csum)
{
	int nleft = len;
	const u_short *w = addr;
	u_short answer;
	int sum =-(csum);
	while (nleft > 1)  {
		sum += *w++;
		nleft -= 2;
	}
	if (nleft == 1)
		sum += htons(*(u_char *)w<<8);

	sum = (sum >> 16) + (sum & 0xffff);	/* add hi 16 to low 16 */
	sum += (sum >> 16);			/* add carry */
	answer = ~sum;				/* truncate to 16 bits */
	return answer;
}



/* for computing TCP/UDP checksums, see TCP/IP Illustrated p. 145 */
u_short 
tcp_cksum(const struct in_addr src, const struct in_addr dst,
				  u_char proto, u_short len,const char *hstart, u_short csum)
{
	struct pseudo {
		struct in_addr src;
		struct in_addr dst;        
		u_char zero;
		u_char proto;        
		u_short length;
	};
	struct pseudo hdr;
	hdr.src = src;
	hdr.dst = dst;
	hdr.zero = 0;
	hdr.proto = proto;
	hdr.length = ntohs(len);
	u_short res=0;
	res = in_cksumip((u_short *) &hdr, sizeof(struct pseudo),0);
	return in_cksumip((u_short *) hstart, len,csum + res);
}



void 
CloseProg (void)
{
if (runmode == CLIENT) {CloseWindows(); return;}
//	pthread_rwlock_destroy(sharedb->rwlock);
	Shm_unlink("/tmp/db.shm");
	Shm_unlink("/tmp/filterdb.shm");
	Shm_unlink("/tmp/sharedb.shm");
	free(readdev);
    free(dbt);
    free(dbb);
    initnode();
    ShutNodes();
    CloseFiles();
	remove_pid(); //remove("/var/run/skyfish.pid");
	if (runmode == SERVER) return;
    CloseWindows();
}


/*
*   Handler timer
*/
void 
callme(int sig_no)
{
	if (runmode == CLIENT) {ShowSpeed();alarm(INTERVAL);return;}
	sharedb->realspeed= sharedb->speed;
    sharedb->speed=0;
	if (runmode != SERVER) ShowSpeed();
    alarm(INTERVAL);
}


/*
 * save statistic in file  -i
 */
void 
InfoProg(int sig_no)
{
	char tm[1024];
		time_t t;
	    time(&t);


Logger("\n");
Logger(ctime(&t));
u_long i;
    for (i=0; i < MAXTHREADS;i++)
		if (db[i].dst > 0 && db[i].refer == MAXTHREADS && db[i].httpf == 1 && db[i].begin ==1)
		{
			bzero(tm,1024);
			sprintf(tm,"|%-4d|%-60s|%5s|%12llu|%12llu|\n",
					i,db[i].name,db[i].type,db[i].len,db[i].lencur);
			Logger(tm);
		}
}



void 
ExitProg(int sig_no)
{
done=0;
}


/*
 *  register real timer for show title programm on the screen
 */
int
RegisterTimer(void)
{
    sigemptyset(&daemonexit.sa_mask);
    sigemptyset(&daemoninfo.sa_mask);
    sigemptyset(&chwindow.sa_mask);
    sigemptyset(&action.sa_mask);
	
	action.sa_handler = (void (*)(int))callme;
    action.sa_flags = SA_RESTART;
	
	chwindow.sa_handler = (void (*)(int))Resize;
    chwindow.sa_flags = SA_RESTART;
    daemoninfo.sa_handler = (void (*)(int))InfoProg;
    daemoninfo.sa_flags = SA_RESTART;
    daemonexit.sa_handler = (void (*)(int))ExitProg;
    daemonexit.sa_flags = SA_RESTART;
    
	if (sigaction(SIGUSR1, &daemonexit, 0) == -1)
    {
    printf("error sigaction\n");
    return 1;
    }
	
	if (sigaction(SIGUSR2, &daemoninfo, 0) == -1)
    {
    printf("error sigaction\n");
    return 1;
    }
    
	if (sigaction(SIGALRM, &action, 0) == -1)
    {
    printf("error sigaction\n");
    return 1;

    }

    if (sigaction(SIGWINCH, &chwindow, 0) == -1)
    {
    printf("error SIGWINCH action\n");
    return 1;
    }
    
    if (alarm(INTERVAL) == -1) {
    printf("error sigalarm\n");
	return 1;
    }
return 0;
}


inline u_short 
checksum(const u_char* data, int len, u_long c)
{
	while(len > 1)
	{	c += *(u_short*)data;
		data += 2; len -= 2;
	}
	if(len) c += *data;

	c = (c >> 16) + (c & 0xffff);
	c += (c >> 16);
	
	return (u_short)(~c);
}

void
TransformDBB(void)
{
    u_long i;
    u_char X;
    u_long n;
    u_long m=0;
    char * ps;
    char * uu;
    u_char tt[100];

    for (i=0; i < DBTSIZE; i++)
	if (strlen(dbb[i].type) > 0) {
	    bzero(tt,100);
	    uu = dbb[i].search;
	    m=0;
	    for (n=0; n<50 && *uu>0; n++) {
		if (*uu == '\\')
		    {
		   uu++;
		sscanf(uu,"%x",&X); 
		tt[m]=X;	    
		m++;
		uu+=2;
		} else {
		tt[m]=*uu;
		m++;
		uu++;
		     }
	    }
	dbb[i].len = m;
	for (m=0; m<dbb[i].len; m++)
	    dbb[i].search[m] = tt[m];
	}

}



inline u_short tcp_checksum3(u_short *addr, int len)
{
    register int nleft = len;
    register u_short *w = addr;
    register int sum = 0;
    u_short answer = 0;

     /* Our algorithm is simple, using a 32 bit accumulator (sum), we add
      * sequential 16 bit words to it, and at the end, fold back all the
      * carry bits from the top 16 bits into the lower 16 bits. */

     while (nleft > 1)  {
         sum += *w++;
         nleft -= 2;
     }

     /* mop up an odd byte, if necessary */
     if (nleft == 1) {
         *(u_char *)(&answer) = *(u_char *) w;
         sum += answer;
     }

     /* add back carry outs from top 16 bits to low 16 bits */
     sum = (sum >> 16) + (sum & 0xffff); /* add hi 16 to low 16 */
     sum += (sum >> 16);                /* add carry */
     answer = ~sum;                     /* truncate to 16 bits */
     return(answer);
}


void
remove_pid(void)
{
	if (runmode == CLIENT) return;
		remove("/var/run/skyfish.pid");
		return ;
}
